﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Text;
using System.Threading.Tasks;
using Windows.Graphics.Imaging;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Xaml.Media.Imaging;

namespace Stencil.Helpers
{
    // Source : https://github.com/XamlBrewer/UWP-SQLite-Sample/blob/master/XamlBrewer.Uwp.SqLiteSample/ByteArrayBitmapExtensions.cs
    public static class ByteArrayBitmapHelper
    {
        public static async Task<byte[]> AsByteArray(this StorageFile file)
        {
            IRandomAccessStream fileStream = await file.OpenAsync(FileAccessMode.Read);
            var reader = new Windows.Storage.Streams.DataReader(fileStream.GetInputStreamAt(0));
            await reader.LoadAsync((uint)fileStream.Size);

            byte[] pixels = new byte[fileStream.Size];

            reader.ReadBytes(pixels);

            return pixels;
        }


        /// <summary>
        /// test : find a good solution
        /// </summary>
        /// <param name="bitmap"></param>
        /// <returns></returns>
        //public static async Task<byte[]> AsByteArray(this BitmapImage image)
        //{
        //}


        public static byte[] AsByteArray(this WriteableBitmap bitmap)
        {
            using (Stream stream = bitmap.PixelBuffer.AsStream())
            {
                MemoryStream memoryStream = new MemoryStream();
                stream.CopyTo(memoryStream);
                return memoryStream.ToArray();
            }
        }


        public static BitmapImage AsBitmapImage(this byte[] byteArray)
        {
            if (byteArray != null)
            {
                using (var stream = new InMemoryRandomAccessStream())
                {
                    stream.WriteAsync(byteArray.AsBuffer()).GetResults(); // I made this one synchronous on the UI thread; this is not a best practice. 
                    var image = new BitmapImage();
                    stream.Seek(0);
                    image.SetSource(stream);
                    return image;
                }
            }


            return null;
        }


        public static BitmapImage AsBitmapImage(this IRandomAccessStream iraStream)
        {
            if (iraStream.Size != 0)
            {
                var image = new BitmapImage();
                iraStream.Seek(0);
                image.SetSource(iraStream);
                return image;
            }

            return null;
        }

        private static async Task<BitmapImage> AsBitmapImage(this StorageFile file)
        {
            var stream = await file.OpenAsync(FileAccessMode.Read);
            var bitmapImage = new BitmapImage();
            await bitmapImage.SetSourceAsync(stream);
            return bitmapImage;
        }

    }
}
