﻿using Syncfusion.UI.Xaml.Diagram;
using Syncfusion.UI.Xaml.Diagram.Controls;
using Syncfusion.UI.Xaml.Diagram.Stencil;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Graphics.Display;
using Windows.Graphics.Imaging;
using Windows.Storage.Streams;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace Stencil
{

    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {

        #region Properties and fields

        private string imageurl;
        public string ImageUrl
        {
            get
            {
                return imageurl;
            }
            set
            {
                if (imageurl != value)
                {
                    if (value != null)
                    {
                        imageurl = value;
                        OnPropertyChanged(("ImageUrl"));
                    }
                }
            }
        }
        private Brush _mFill = new SolidColorBrush(Windows.UI.Colors.Transparent);
        public Brush Fill
        {
            get
            {
                return _mFill;
            }
            set
            {
                if (_mFill != value)
                {
                    _mFill = value;
                    OnPropertyChanged("Fill");
                }
            }
        }

        private Brush _mstroke = new SolidColorBrush(Windows.UI.Colors.Transparent);
        public Brush Stroke
        {
            get
            {
                return _mstroke;
            }
            set
            {
                if (_mstroke != value)
                {
                    _mstroke = value;
                    OnPropertyChanged("Stroke");
                }
            }
        }
        private double _Thickness = 4;
        public double Thickness
        {
            get
            {
                return _Thickness;

            }
            set
            {
                if (value != _Thickness)
                {
                    _Thickness = value;
                    OnPropertyChanged("Thickness");
                }
            }
        }

        Style style = null;

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(name));
            }
        }

        #endregion

        #region Cstor

        public MainPage()
        {
            this.InitializeComponent();

            //diagram.SFSelector.Style = this.Resources["CustomSelector"] as Style;

            //PCD : test => load properties lists
            LoadColors();
            //LoadThicknessSizes();
            style = new Style();
            diagram.PortVisibility = PortVisibility.Visible;

            //PCD : test => block the image properties
            DiagramCollection<NodeViewModel> nodes = (diagram as SfDiagram).Nodes as DiagramCollection<NodeViewModel>;
            if (nodes != null && nodes.Count > 0)
            {
                int cptNodes = 0;
                foreach (var node in nodes)
                {
                    cptNodes++;
                    if (cptNodes == 1)
                    {
                        node.Constraints = ~NodeConstraints.Rotatable & ~NodeConstraints.Menu &
                                            ~NodeConstraints.Resizable & ~NodeConstraints.Draggable &
                                            ~NodeConstraints.OutConnect & ~NodeConstraints.Connectable &
                                            ~NodeConstraints.Selectable;
                        node.ZIndex = 0;
                     
                    }
                }
            }


            //PCD : test => page properties and layout
            /*
            diagram.Constraints = diagram.Constraints & ~GraphConstraints.PanRails;
            diagram.Constraints |= GraphConstraints.Bridging;
            */

            //To Disable ContextMenu
            diagram.Menu = null;
            //Initialize PageSettings and Snapping 
            InitializeDiagram();

            //diagram.PointerReleased += diagramcontrol_PointerReleased;
            //diagram.Tool = Tool.ZoomPan;

            // Syncfusion => disable selection area
            diagram.PageSettings.PageBorderBrush = new SolidColorBrush(Windows.UI.Colors.Transparent);

            (diagram.Info as IGraphInfo).ItemAdded += MainWindow_ItemAdded;
            (diagram.SelectedItems as SelectorViewModel).Constraints &= ~NodeConstraints.OutConnect & ~NodeConstraints.Connectable
                                                                     & ~NodeConstraints.InConnect;
            //(diagram.SelectedItems as SelectorViewModel).SelectorConstraints &= ~SelectorConstraints.QuickCommands;
            stencil.SelectedFilter = new SymbolFilterProvider() { SymbolFilter = SymbolFilter };
        }

        #endregion

        #region Private

        private void InitializeDiagram()
        {
            diagram.ScrollSettings.ScrollLimit = ScrollLimit.Diagram;
            //diagram.ScrollSettings.ScrollLimit = ScrollLimit.Limited;
            diagram.PageSettings.PageBorderBrush = new SolidColorBrush(Windows.UI.Colors.Transparent);
            diagram.PageSettings.PageBackground = new SolidColorBrush(Windows.UI.Colors.Transparent);
        }

        private void MainWindow_ItemAdded(object sender, ItemAddedEventArgs args)
        {
            if (args.ItemSource == ItemSource.Stencil)
            {
                var dropedItem = args.Item as NodeViewModel;

                if (dropedItem != null)
                {
                    dropedItem.ShapeStyle = getstyle();
                }
            }
        }

        //Filter event
        private bool SymbolFilter(SymbolFilterProvider sender, object symbol)
        {
            return true;
        }

        #region -> Style 

        private Style GetCustomStyle()
        {
            Style s = new Style();
            s.TargetType = typeof(Windows.UI.Xaml.Shapes.Path);
            if (Fill!=null )
            {
                s.Setters.Add(new Setter(Windows.UI.Xaml.Shapes.Path.FillProperty, Fill));

            }
            if (!double.IsNaN(Thickness) && !double.IsInfinity(Thickness))
            {
                s.Setters.Add(new Setter(Windows.UI.Xaml.Shapes.Path.StrokeThicknessProperty, Thickness));
            }
            if (Stroke != null)
            {
                s.Setters.Add(new Setter(Windows.UI.Xaml.Shapes.Path.StrokeProperty, Stroke));
            }
            s.Setters.Add(new Setter(Windows.UI.Xaml.Shapes.Path.StretchProperty, Stretch.Fill));

            return s;
        }

        private Style getstyle()
        {
            Style s = this.Resources["shapeStyle1"] as Style;
            style.TargetType = typeof(Windows.UI.Xaml.Shapes.Path);
            if (style.Setters.Count() == 0)
            {
                foreach (Setter set in s.Setters)
                {
                    if (set.Property == Windows.UI.Xaml.Shapes.Path.FillProperty)
                    {
                        Fill = (Brush)set.Value;
                    }
                    if (set.Property == Windows.UI.Xaml.Shapes.Path.StrokeProperty)
                    {
                        Stroke = (Brush)set.Value;
                    }
                    if (set.Property == Windows.UI.Xaml.Shapes.Path.StrokeThicknessProperty)
                    {
                        Thickness = (double)set.Value;
                    }

                }
            }

            return style;
        }

        #endregion

        #region -> Color

        public void LoadColors()
        {
            List<Colors> lColors = new List<Colors>();
            Type type = typeof(Windows.UI.Colors);
            PropertyInfo[] Properties = type.GetRuntimeProperties().ToArray<PropertyInfo>();
            foreach (var item in Properties)
            {
                string s = item.Name;
                Windows.UI.Color c = (Windows.UI.Color)item.GetValue(null, null);
                Colors newcolor = new Colors();
                lColors.Add(new Colors()
                {
                    Color = item.Name,
                    ColorBrush = new SolidColorBrush(c)
                });
            }
            colorList.ItemsSource = lColors;
        }
        Colors color = null;
        private void colorList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            color = (Colors)e.AddedItems[0];
            if (diagram.Nodes != null)
            {
                foreach (NodeViewModel node in diagram.Nodes as DiagramCollection<NodeViewModel>)
                {
                    if (node.IsSelected)
                    {
                        if (node.Key != null)
                        {
                            if (thick != 0)
                                Thickness = thick;
                            if (node.Key.ToString().ToLower().Contains("arrow"))
                            {
                                Fill = color.ColorBrush;
                                Stroke = color.ColorBrush;
                                node.ShapeStyle = GetCustomStyle();
                            }
                            else
                            {
                                Stroke = color.ColorBrush;
                                Fill = null;
                                node.ShapeStyle = GetCustomStyle();
                            }
                        }
                    }
                }
            }
            //myGrid.Background = color.ColorBrush;
        }

        #endregion

        #region -> Thickness
        /*
        public void LoadThicknessSizes()
        {
            List<int> lSizes = new List<int>();

            for (int i = 1; i <= 10; i++)
            {
                lSizes.Add(i);
            }
            thicknessSizeList.ItemsSource = lSizes;
        }
        private void thicknessSizeList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var color = (int)e.AddedItems[0];
            if (diagram.Nodes != null)
            {
                foreach (NodeViewModel node in diagram.Nodes as DiagramCollection<NodeViewModel>)
                {
                    if (node.IsSelected)
                    {
                        Thickness = color;
                        node.ShapeStyle = GetCustomStyle();
                    }
                }
            }
        }
        */
        int thick = 0;
        private void thicknessSizeSlider_ValueChanged(object sender, RangeBaseValueChangedEventArgs e)
        {
             thick = (int)e.NewValue;
            if (diagram != null && diagram.Nodes != null)
            {
                foreach (NodeViewModel node in diagram.Nodes as DiagramCollection<NodeViewModel>)
                {
                    if (node.IsSelected)
                    {                    
                        Thickness = thick;
                        if (color != null && node.Key.ToString().ToLower().Contains("arrow"))
                        {
                            Fill = (color as Colors).ColorBrush;
                            Stroke = (color as Colors).ColorBrush;
                        }
                        else
                            Fill = null;
                        node.ShapeStyle = GetCustomStyle();
                    }
                }
            }
        }

        #endregion

        #region -> Save image

        private async void Button_Click(object sender, RoutedEventArgs e)
        {

            var result = await GenerateImage(diagram);
            Frame.Navigate(typeof(ResultPage), result);
        }

        /// <summary>
        /// Save as image
        /// (https://www.syncfusion.com/forums/126075/export-uwp-diagram-to-image)
        /// </summary>
        /// <param name="diagram"></param>
        /// <returns></returns>
        protected async Task<IRandomAccessStream> GenerateImage(UIElement diagram)
        {
            // Render to an image at the current system scale and retrieve pixel contents
            var renderTargetBitmap = new RenderTargetBitmap();

            // unselect all nodes
            //ObservableCollection<TraNode> nodes = ((diagram as SfDiagram).Nodes as ObservableCollection<TraNode>);
            //ObservableCollection<Node> nodes = ((diagram as SfDiagram).Nodes as ObservableCollection<Node>);
            /*
            if (nodes != null && nodes.Count > 0)
            {
                foreach (var node in nodes)
                {
                    node.IsSelected = false;
                }
            }
            */
            DiagramCollection<NodeViewModel> nodes = (diagram as SfDiagram).Nodes as DiagramCollection<NodeViewModel>;
            if (nodes != null && nodes.Count > 0)
            {
                foreach (var node in nodes)
                {
                    node.IsSelected = false;
                }
            }


            await renderTargetBitmap.RenderAsync(diagram);
            var pixelBuffer = await renderTargetBitmap.GetPixelsAsync();

            var pixelBytes = new byte[pixelBuffer.Length];
            using (var reader = DataReader.FromBuffer(pixelBuffer))
            {
                reader.ReadBytes(pixelBytes);
            }

            IRandomAccessStream stream = new InMemoryRandomAccessStream();
            //var encoder = await BitmapEncoder.CreateAsync(BitmapEncoder.PngEncoderId, stream);
            var encoder = await BitmapEncoder.CreateAsync(BitmapEncoder.JpegEncoderId, stream);

            //1920 x 1080 = ratio de 1.777
            var resHeight = renderTargetBitmap.PixelWidth / 1.777;

            encoder.SetPixelData(
                BitmapPixelFormat.Bgra8,
                BitmapAlphaMode.Straight,
                (uint)renderTargetBitmap.PixelWidth,
                 //(uint)renderTargetBitmap.PixelHeight,
                 (uint)resHeight,
                DisplayInformation.GetForCurrentView().LogicalDpi,
                DisplayInformation.GetForCurrentView().LogicalDpi,
                pixelBytes);

            await encoder.FlushAsync();

            return stream;
        }

        #endregion

        #endregion
    }

    public class NodeVM : NodeViewModel
    {

        private string _image = "ms-appx:///Assets/Image.jpeg";

        public string CustomImage
        {
            get { return _image; }
            set
            {
                if (_image != value)
                {
                    _image = value;
                }
            }
        }
    }
    public class PortCollection : ObservableCollection<IPort>
    {

    }
    //Collection of Symbols
    public class SymbolCollection : ObservableCollection<object>
    {

    }
    public class Colors
    {
        public string Color
        {
            get;
            set;
        }

        public Brush ColorBrush
        {
            get;
            set;
        }
    }

}
